/*  tds200mt
 *  Copyright (C) 1999 Alvaro De Amicis
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <math.h>
#include <time.h>
#include "tmt.h"
#include "comm.h"

void
cl_help()
{
	printf("tmt %s\n", VERSION);
	printf("usage: tmt [-a SOURCE [-o FILE]] [-p FILE]\n");
	printf("-a --acquire=SOURCE           transfer waveform data from TDS, \n");
    printf("                              SOURCE specifies the location of\n"); 
    printf("                              the data, correct values are: \n");
    printf("                              ch1, ch2, math, refa, refb   or\n");
    printf("                              1, 2, m, a, b\n");
    printf("-c --comm=PORT                use serial port PORT. values are: \n");
    printf("                              0, 1, 2, 3    (ttyS0-3)\n");
    printf("-o --out=FILE                 writes data on FILE. If not \n");
    printf("                              specified, uses `TDSWaveData'.\n"); 
    printf("-p --print=FILE               print waveform. Gnuplot is required.\n");
    printf("-h --help                     display this help and exit.\n");
	printf("\n");
	printf("Report bugs to a.deamicis@iol.it\n");
}


int
cl_print(char *filename)
{
	TdsWave *twave = (TdsWave*) malloc(sizeof(TdsWave));
	FILE *fp;
	int i;
	
	if ((fp = fopen (filename, "r")) != NULL) {
		fread (twave, sizeof(*twave), 1, fp);
		fclose (fp);
	} else
		perror("fopen");
	if ( strncmp ("\"Ch1,", twave->WFMPreWFid, 5) &&
			strncmp ("\"Ch2,", twave->WFMPreWFid, 5) &&
			strncmp ("\"RefA,", twave->WFMPreWFid, 6) &&
			strncmp ("\"RefB,", twave->WFMPreWFid, 6) &&
			strncmp ("\"Math,", twave->WFMPreWFid, 6)
			) {
		fprintf (stderr, "Error: %s isn't a valid file!\n", filename);
		return 1;
	}

	desctoid(twave->WFMPreWFid, twave->ID);

	if ((fp = fopen ("source.tmp", "w")) != NULL) {
		for (i = 0; i<2500; i++)
			fprintf (fp, "%i\n", twave->WFMdata[i]);
		fclose (fp);
	} else {
		perror("fopen");
		return 1;
	}
	if ((fp = fopen ("plot.tmp", "w")) != NULL) {
		fprintf (fp, "set format ''\n");
		fprintf (fp, "set xtics border mirror norotate 250\n");
		fprintf (fp, "set ytics border mirror norotate 25.6\n");
		fprintf (fp, "set grid\n");
		fprintf (fp, "set nokey\n");
		fprintf (fp, "set xlabel '");
		fprintf (fp, "%s", twave->ID);
		fprintf (fp, "'\n");
		fprintf (fp, "set term postscript\n");
		fprintf (fp, "set output 'out.ps'\n");
		fprintf (fp, "plot[0:2500][0:255] 'source.tmp' w l, 128 w l\n");
		fprintf (fp, "set output '|lpr out.ps'\n");
		fprintf (fp, "quit");
		fclose (fp);
	} else {
		perror("fopen");
		fprintf (stderr, "Error: can't write temp file.\n");
	}
	if (system("gnuplot plot.tmp"))
		perror("system");
	else
		printf ("Output sent to gnuplot | lpr\n");
	unlink ("source.tmp");
	unlink ("plot.tmp");
	return 0;
}


int
cl_acquire(Location source, char *filename)
{
	TdsWave *twave = (TdsWave*) malloc(sizeof(TdsWave));
	rxwaveform (twave, source, FALSE);
	if (filename != NULL) {
		FILE *fp;
		if ((fp = fopen (filename, "w")) != NULL) {
			fwrite (twave, sizeof(*twave), 1, fp);
			fclose (fp);
		} else
			perror("fopen");
			return -1;
	}		
	return 0;
}


int
desctoid (char s[], char id[])
{
	float mv, ms;
	int   ev, es;
	char  sv[10], ss[10], tmp[4]="000";
	int   i, q, n;
	
	strncpy (id, &s[1], 4);
	n = strlen (s);
	q = 0;
	id[4] = '\0';			/* end of string */
	
	for ( i = 0; q == 2; i++ ) {
		if ( !strncmp (",", &s[i], 1) )  q++ ;
		if ( i > n ) return -1;
	}

	for ( ; ; i++ ) {
		if ( !strncmp ("E", &s[i], 1) ) {
			strncpy (tmp, &s[i-3], 3); 
			mv = atof (tmp);
			ev = atoi (&s[i+1]);
			i++;
			break;
		}
		if ( i > n ) return -1;
	}

	for ( ; ; i++ ) {
		if ( !strncmp ("E", &s[i], 1) ) {
			strncpy (tmp, &s[i-3], 3);
			ms = atof (tmp);
			es = abs (atoi (&s[i+1]));
			break;
		}
		if ( i > n ) return -1;
	}
	
	switch (ev) {
		case 1:
			sprintf (sv, " %2.0f", mv * 10);
			strcat (sv, "V");
			break; 	
		case 0:	
			sprintf (sv, " %1.1f", mv);
			strcat (sv, "V");
			break; 	
		case -1:	
			sprintf (sv, " %3.0f", mv * 100);
			strcat (sv, "mV");
			break; 	
		case -2:
			sprintf (sv, " %2.0f", mv * 10);
			strcat (sv, "mV");
			break;
		default:
			return -1; 	
	}

	switch (es) {
		case 0:
			sprintf (ss, " %1.1f", ms);
			strcat (ss, "s");
			break; 	
		case 1:
		case 2:
		case 3:	
			sprintf (ss, " %3.1f", ms * (1000/pow(10,es)));
			strcat (ss, "ms");
			break; 	
		case 4:
		case 5:
		case 6:	
			sprintf (ss, " %3.1f", ms * (1000/pow(10,es-3)));
			strcat (ss, "us");
			break; 	
		case 7:
		case 8:
			sprintf (ss, " %3.0f", ms * (1000/pow(10,es-6)));
			strcat (ss, "ns");
			break;
		default:
			return -1; 
	}
	strcat(id, sv);
	strcat(id, ss);

	return 0;
}

void 
delay(int sec)
{
	time_t t = time(NULL);
	while ( time(NULL) - t < sec ) { }
}

